@@ -1,7 +1,7 @@
 /* ignuit - Educational software for the GNOME, following the Leitner
  * flash-card system.
  *
- * Copyright (C) 2008, 2009 Timothy Richard Musson
+ * Copyright (C) 2008, 2009, 2012 Timothy Richard Musson
  *
  * Email: <trmusson@gmail.com>
  * WWW:   http://homepages.ihug.co.nz/~trmusson/programs.html#ignuit
@@ -1097,11 +1097,12 @@ error_dialog (GtkWindow *parent, const gchar *message, GError *err)
     else
         msg = g_strdup (message);
 
-    dialog = (GtkDialog*)gtk_message_dialog_new_with_markup (parent,
+    dialog = (GtkDialog*)gtk_message_dialog_new (parent,
         GTK_DIALOG_MODAL,
         GTK_MESSAGE_ERROR,
         GTK_BUTTONS_CLOSE,
-        msg);
+        NULL);
+    gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (dialog), msg);
 
     gtk_dialog_run (dialog);
     gtk_widget_destroy (GTK_WIDGET(dialog));
@@ -1202,11 +1203,12 @@ ask_yes_or_no (GtkWindow *parent, const gchar *question,
     GtkDialog *dialog;
     gint result;
 
-    dialog = (GtkDialog*)gtk_message_dialog_new_with_markup (parent,
+    dialog = (GtkDialog*)gtk_message_dialog_new (parent,
         GTK_DIALOG_MODAL,
         GTK_MESSAGE_QUESTION,
         GTK_BUTTONS_YES_NO,
-        question);
+        NULL);
+    gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG (dialog), question);
 
     gtk_dialog_set_default_response (dialog, def);
     result = gtk_dialog_run (dialog);
@@ -1314,6 +1316,7 @@ cb_m_new (GtkWidget *widget, AppWin *d)
     app_window_update_sensitivity (d);
 
     dialog_editor_tweak (ED_TWEAK_ALL);
+    dialog_properties_tweak ();
 }
 
 
@@ -1387,6 +1390,7 @@ cb_m_open (GtkWidget *widget, AppWin *d)
 
             app_window_update_title (d->ig);
             dialog_editor_tweak (ED_TWEAK_ALL);
+            dialog_properties_tweak ();
 
             if (err != NULL) {
 
@@ -3339,6 +3343,7 @@ app_window (Ignuit *ig)
     g_signal_connect (G_OBJECT(d->ig->treev_cat), "drag_data_received",
         G_CALLBACK(cb_treev_categories_drag_data_received), d);
 
+
     app_window_refresh_category_pane (d->ig);
     app_window_refresh_card_pane (d->ig,
         file_get_current_category_cards (d->ig->file));
