@@ -380,7 +380,14 @@ cb_window_key_press_event (GtkWindow *w, GdkEventKey *event, Dialog *d)
 {
     gboolean handled;
 
-    /* This is necessary to prevent copy/paste key presses intended
+#if 0
+    if (event->state & GDK_CONTROL_MASK && event->keyval == GDK_s) {
+        /* XXX: Save the file. */
+        return TRUE;
+    }
+#endif
+
+    /* The following is necessary to prevent copy/paste key presses intended
      * for the focused tag bar from ending up in the textview.
      * (This solution found in gedit 2.28.0, gedit-window.c) */
 
@@ -1497,6 +1504,16 @@ dialog_editor_card_style_changed (void)
 }
 
 
+gboolean
+cb_key_press (GtkWidget *w, GdkEventKey *event, gpointer data)
+{
+    if (event->keyval == GDK_Escape) {
+        g_print ("beep\n");
+    }
+    return FALSE;
+}
+
+
 void
 dialog_editor (Ignuit *ig)
 {
