@@ -1,7 +1,7 @@
 /* ignuit - Educational software for the GNOME, following the Leitner
  * flash-card system.
  *
- * Copyright (C) 2008, 2009 Timothy Richard Musson
+ * Copyright (C) 2008, 2009, 2012 Timothy Richard Musson
  *
  * Email: <trmusson@gmail.com>
  * WWW:   http://homepages.ihug.co.nz/~trmusson/programs.html#ignuit
@@ -216,6 +216,16 @@ cb_close (GtkWidget *widget, Dialog *d)
 }
 
 
+void
+dialog_properties_tweak (void)
+{
+    if (dialog == NULL)
+        return;
+
+    set_widget_values (dialog);
+}
+
+
 void
 dialog_properties (Ignuit *ig)
 {
