@@ -57,6 +57,8 @@ struct _QuizItem {
     gboolean known;
     gboolean reverse;
 
+    gchar *note;
+
 };
 
 
@@ -71,17 +73,22 @@ typedef struct {
 
     GList               *cardview_selected;
 
+    GtkWidget           *hbox_answer_bar;
+    GtkWidget           *entry_answer_bar;
+    GtkWidget           *b_clear_answer_bar;
+
     GtkWidget           *m_prev;
     GtkWidget           *m_next;
     GtkCheckMenuItem    *m_known;
     GtkCheckMenuItem    *m_unknown;
     GtkCheckMenuItem    *m_flip;
     GtkCheckMenuItem    *m_info;
+    GtkCheckMenuItem    *m_answer_bar;
     GtkWidget           *m_listen_front;
     GtkWidget           *m_listen_back;
     GtkCheckMenuItem    *m_auto_listen_front;
     GtkCheckMenuItem    *m_auto_listen_back;
-    GtkCheckMenuItem    *m_tag;
+    GtkCheckMenuItem    *m_flag;
     GtkToggleButton     *b_info;
     GtkWidget           *b_prev;
     GtkWidget           *b_next;
@@ -107,7 +114,7 @@ typedef struct {
 } Dialog;
 
 
-static void cb_m_tag (GtkCheckMenuItem *widget, Dialog *d);
+static void cb_m_flag (GtkCheckMenuItem *widget, Dialog *d);
 static void toggle_ui_known (Dialog *d, gboolean active);
 static void toggle_ui_unknown (Dialog *d, gboolean active);
 static void toggle_ui_info (Dialog *d, gboolean active);
@@ -130,6 +137,26 @@ quizitem_new (Card *card)
 }
 
 
+static void
+quizitem_free (QuizItem *item)
+{
+    g_free (item->note);
+    g_free (item);
+}
+
+
+static void
+quizitem_set_note (Dialog *d)
+{
+    QuizItem *item;
+
+    item = QUIZITEM(d->cur);
+
+    g_free (item->note);
+    item->note = g_strdup (gtk_entry_get_text (GTK_ENTRY(d->entry_answer_bar)));
+}
+
+
 static GList*
 shuffle_quizitems (GRand *grand, GList *list)
 {
@@ -138,9 +165,8 @@ shuffle_quizitems (GRand *grand, GList *list)
     gpointer data;
 
 
-    if (g_list_length (list) < 2) {
+    if (g_list_length (list) < 2)
         return list;
-    }
 
     while ((len = g_list_length (list)) > 0) {
         x = g_rand_int_range (grand, 0, len);
@@ -178,8 +204,8 @@ quizitems_from_cards (GList *list, QuizInfo *info)
             add_it = (card_get_group (CARD(cur)) == GROUP_NEW ||
                 card_get_expired (CARD(cur)));
             break;
-        case QUIZ_TAGGED_CARDS:
-            add_it = card_get_tagged (CARD(cur));
+        case QUIZ_FLAGGED_CARDS:
+            add_it = card_get_flagged (CARD(cur));
             break;
         case QUIZ_SELECTED_CARDS:
             add_it = TRUE;
@@ -189,11 +215,9 @@ quizitems_from_cards (GList *list, QuizInfo *info)
             break;
         }
 
-        if (add_it) {
-
+        if (add_it)
             quizitems = g_list_append (quizitems, quizitem_new (cur->data));
 
-        }
     }
     return quizitems;
 }
@@ -219,9 +243,8 @@ get_quizitems (Dialog *d, GRand *grand)
         switch (d->ig->quizinfo.category_selection) {
         case QUIZ_CURRENT_CATEGORY:
             cat = file_get_current_category (d->ig->file);
-            if (category_get_n_cards (cat) > 0) {
+            if (category_get_n_cards (cat) > 0)
                 cards = category_get_cards (cat);
-            }
             break;
         case QUIZ_ALL_CATEGORIES:
             cards = file_get_cards (d->ig->file);
@@ -240,14 +263,12 @@ get_quizitems (Dialog *d, GRand *grand)
 
     switch (d->ig->quizinfo.face_selection) {
     case QUIZ_FACE_BACK:
-        for (cur = quizitems; cur != NULL; cur = cur->next) {
+        for (cur = quizitems; cur != NULL; cur = cur->next)
             QUIZITEM(cur)->reverse = TRUE;
-        }
         break;
     case QUIZ_FACE_RANDOM:
-        for (cur = quizitems; cur != NULL; cur = cur->next) {
+        for (cur = quizitems; cur != NULL; cur = cur->next)
             QUIZITEM(cur)->reverse = g_rand_int_range (grand, 0, 2);
-        }
         break;
     default:
         break;
@@ -269,19 +290,15 @@ store_quiz_results (Dialog *d, gboolean store_statistics)
 
         if (QUIZITEM(cur)->tested) {
 
-            if (store_statistics) {
-
+            if (store_statistics)
                 card_mark_tested (d->ig->prefs, QCARD(cur),
                         &d->date_tested, d->time_tested, QUIZITEM(cur)->known);
 
-            }
-
-            /* Move unknown cards to the top in their categories. */
-            if (!QUIZITEM(cur)->known) {
+            /* Move cards marked as "unknown" to the top in their categories. */
+            if (!QUIZITEM(cur)->known)
                 card_to_top (QCARD(cur));
-            }
 
-            file_set_changed (d->ig->file, TRUE);
+            ig_file_changed (d->ig);
 
         }
     }
@@ -292,9 +309,6 @@ store_quiz_results (Dialog *d, gboolean store_statistics)
         file_get_current_category_cards (d->ig->file));
 
     app_window_update_appbar (d->ig);
-
-    /* Add an asterisk to the main window title if the file has changed. */
-    app_window_update_title (d->ig);
 }
 
 
@@ -384,30 +398,36 @@ show_current_card (Dialog *d, gboolean quiet)
 
     active = QUIZITEM(d->cur)->tested && QUIZITEM(d->cur)->known;
     toggle_ui_known (d, active);
-    if (active) {
+    if (active)
         color = prefs_get_color (d->ig->prefs, COLOR_CARD_BG_KNOWN);
-    }
 
     active = QUIZITEM(d->cur)->tested && !QUIZITEM(d->cur)->known;
     toggle_ui_unknown (d, active);
-    if (active) {
+    if (active)
         color = prefs_get_color (d->ig->prefs, COLOR_CARD_BG_UNKNOWN);
-    }
 
     gtk_widget_modify_base (GTK_WIDGET(d->textview[FRONT]),
         GTK_STATE_NORMAL, color);
     gtk_widget_modify_base (GTK_WIDGET(d->textview[BACK]),
         GTK_STATE_NORMAL, color);
 
-    g_signal_handlers_block_by_func (G_OBJECT(d->m_tag),
-        G_CALLBACK(cb_m_tag), d);
-    gtk_check_menu_item_set_active (d->m_tag, card_get_tagged (c));
-    g_signal_handlers_unblock_by_func (G_OBJECT(d->m_tag),
-        G_CALLBACK(cb_m_tag), d);
+    g_signal_handlers_block_by_func (G_OBJECT(d->m_flag),
+        G_CALLBACK(cb_m_flag), d);
+    gtk_check_menu_item_set_active (d->m_flag, card_get_flagged (c));
+    g_signal_handlers_unblock_by_func (G_OBJECT(d->m_flag),
+        G_CALLBACK(cb_m_flag), d);
 
-    if (!quiet && (prefs_get_auto_listen (d->ig->prefs) & LISTEN_FRONT)) {
+    update_title ();
+
+    if (QUIZITEM(d->cur)->note != NULL)
+        gtk_entry_set_text (GTK_ENTRY(d->entry_answer_bar),
+            QUIZITEM(d->cur)->note);
+    else
+        gtk_entry_set_text (GTK_ENTRY(d->entry_answer_bar), "");
+
+    if (!quiet && (prefs_get_auto_listen (d->ig->prefs) & LISTEN_FRONT))
         audio_play_side (FRONT);
-    }
+
 }
 
 
@@ -433,7 +453,10 @@ set_interface_sensitivity (Dialog *d)
     gtk_widget_set_sensitive (GTK_WIDGET(d->m_known), !last);
     gtk_widget_set_sensitive (GTK_WIDGET(d->m_flip), !last);
     gtk_widget_set_sensitive (GTK_WIDGET(d->m_info), !last);
-    gtk_widget_set_sensitive (GTK_WIDGET(d->m_tag), !last);
+    gtk_widget_set_sensitive (GTK_WIDGET(d->m_flag), !last);
+
+    gtk_widget_set_sensitive (GTK_WIDGET(d->entry_answer_bar), !last);
+    gtk_widget_set_sensitive (GTK_WIDGET(d->b_clear_answer_bar), !last);
 
     gtk_widget_set_sensitive (GTK_WIDGET(d->chk_store),
         d->n_quizitems > 0);
@@ -447,6 +470,8 @@ step_forward (Dialog *d)
 
     if (d->cur->next) {
 
+        quizitem_set_note (d);
+
         d->cur = d->cur->next;
         d->n = d->n + 1;
 
@@ -470,6 +495,8 @@ step_back (Dialog *d)
 
     if (d->cur->prev) {
 
+        quizitem_set_note (d);
+
         d->cur = d->cur->prev;
         d->n = d->n - 1;
 
@@ -501,20 +528,63 @@ cb_m_info (GtkCheckMenuItem *widget, Dialog *d)
     active =  gtk_check_menu_item_get_active (widget);
     toggle_ui_info (d, active);
 
-    if (active) {
+    if (active)
         page = INFO;
-    }
-    else if (gtk_check_menu_item_get_active (d->m_flip)) {
+    else if (gtk_check_menu_item_get_active (d->m_flip))
         page = BACK;
-    }
-    else {
+    else
         page = FRONT;
-    }
    
     gtk_notebook_set_current_page (d->notebook, page);
 }
 
 
+static void
+cb_m_answer_bar (GtkCheckMenuItem *widget, Dialog *d)
+{
+    gboolean active;
+
+    active =  gtk_check_menu_item_get_active (widget);
+    if (active) {
+        gtk_widget_show (d->hbox_answer_bar);
+        gtk_widget_grab_focus (d->entry_answer_bar);
+    }
+    else {
+        gtk_widget_hide (d->hbox_answer_bar);
+    }
+
+    prefs_set_quiz_answer_bar_visible (d->ig->prefs, active);
+}
+
+
+static void
+cb_b_close_answer_bar (GtkWidget *widget, Dialog *d)
+{
+    gtk_check_menu_item_set_active (d->m_answer_bar, FALSE);
+    gtk_widget_hide (d->hbox_answer_bar);
+
+    prefs_set_quiz_answer_bar_visible (d->ig->prefs, FALSE);
+}
+
+
+static void
+cb_b_clear_answer_bar (GtkWidget *widget, Dialog *d)
+{
+    gtk_entry_set_text (GTK_ENTRY(d->entry_answer_bar), "");
+}
+
+
+static void
+cb_answer_bar_activate (GtkWidget *widget, Dialog *d)
+{
+    gboolean flipped;
+
+    flipped = gtk_check_menu_item_get_active (d->m_flip);
+    gtk_check_menu_item_set_active (d->m_flip, !flipped);
+    gtk_widget_grab_focus (d->entry_answer_bar);
+}
+
+
 static void
 cb_b_info (GtkToggleButton *widget, Dialog *d)
 {
@@ -536,12 +606,10 @@ static void
 cb_b_prev (GtkWidget *widget, Dialog *d)
 {
     cb_m_prev (widget, d);
-    if (d->cur->prev) {
+    if (d->cur->prev)
         gtk_widget_grab_focus (widget);
-    }
-    else {
+    else
         gtk_widget_grab_focus (d->b_next);
-    }
 }
 
 
@@ -583,9 +651,8 @@ cb_m_unknown (GtkCheckMenuItem *tb, Dialog *d)
 
     /* If the card was marked as untested, move on to the next card. */
 
-    if (!marked_as_tested) {
+    if (!marked_as_tested)
         step_forward (d);
-    }
 
     show_current_card (d, marked_as_tested);
 }
@@ -622,9 +689,8 @@ cb_m_known (GtkCheckMenuItem *tb, Dialog *d)
 
     /* If the card was marked as untested, move on to the next card. */
 
-    if (!marked_as_tested) {
+    if (!marked_as_tested)
         step_forward (d);
-    }
 
     show_current_card (d, marked_as_tested);
 }
@@ -652,12 +718,11 @@ cb_m_flip (GtkCheckMenuItem *widget, Dialog *d)
     gtk_notebook_set_current_page (d->notebook, page);
     toggle_ui_info (d, FALSE);
 
-    if (page == FRONT && (prefs_get_auto_listen (d->ig->prefs) & LISTEN_FRONT)) {
+    if (page == FRONT && (prefs_get_auto_listen (d->ig->prefs) & LISTEN_FRONT))
         audio_play_side (FRONT);
-    }
-    else if (page == BACK && (prefs_get_auto_listen (d->ig->prefs) & LISTEN_BACK)) {
+    else if (page == BACK && (prefs_get_auto_listen (d->ig->prefs) & LISTEN_BACK))
         audio_play_side (BACK);
-    }
+
 }
 
 
@@ -671,7 +736,7 @@ cb_b_flip (GtkToggleButton *widget, Dialog *d)
 
 
 static void
-cb_m_tag (GtkCheckMenuItem *widget, Dialog *d)
+cb_m_flag (GtkCheckMenuItem *widget, Dialog *d)
 {
     gboolean state;
     Card *c;
@@ -681,20 +746,19 @@ cb_m_tag (GtkCheckMenuItem *widget, Dialog *d)
 
     state = gtk_check_menu_item_get_active (widget);
 
-    if (card_get_tagged (c) != state) {
+    if (card_get_flagged (c) != state) {
 
         GtkTreeIter iter;
         GList *item;
 
-        card_set_tagged (c, state);
-        file_set_changed (d->ig->file, TRUE);
+        card_set_flagged (c, state);
+        ig_file_changed (d->ig);
 
         /* If the card is listed in the main window, update that row. */
 
         item = app_window_find_treev_iter_with_data (d->ig->treev_card, &iter, c);
-        if (item != NULL) {
+        if (item != NULL)
             app_window_refresh_card_row (d->ig, item);
-        }
 
         app_window_update_title (d->ig);
     }
@@ -708,12 +772,10 @@ cb_m_listen (GtkWidget *widget, Dialog *d)
 {
     gint page;
 
-    if (widget == d->m_listen_front) {
+    if (widget == d->m_listen_front)
         page = FRONT;
-    }
-    else {
+    else
         page = BACK;
-    }
 
     audio_play_side (page);
 }
@@ -724,12 +786,12 @@ cb_m_auto_listen (GtkCheckMenuItem *widget, Dialog *d)
 {
     AutoListen listen = 0;
 
-    if (gtk_check_menu_item_get_active (d->m_auto_listen_front)) {
+    if (gtk_check_menu_item_get_active (d->m_auto_listen_front))
         listen |= LISTEN_FRONT;
-    }
-    if (gtk_check_menu_item_get_active (d->m_auto_listen_back)) {
+
+    if (gtk_check_menu_item_get_active (d->m_auto_listen_back))
         listen |= LISTEN_BACK;
-    }
+
     prefs_set_auto_listen (d->ig->prefs, listen);
 }
 
@@ -743,7 +805,8 @@ dialog_quiz_set_sensitive (gboolean sensitive)
      * be to call audio_stop() inside step_forward() and step_back().
      */
 
-    if (dialog == NULL) { return; }
+    if (dialog == NULL)
+        return;
 
     gtk_widget_set_sensitive (dialog->m_prev, sensitive);
     gtk_widget_set_sensitive (dialog->m_next, sensitive);
@@ -775,14 +838,13 @@ window_close (Dialog *d)
     last = d->last;
     card_free (last->card);
 
-    for (cur = d->quizitems; cur != NULL; cur = cur->next) {
-        g_free (cur->data);
-    }
+    for (cur = d->quizitems; cur != NULL; cur = cur->next)
+        quizitem_free ((QuizItem*)cur->data);
+
     g_list_free (d->quizitems);
 
-    if (d->cardview_selected) {
+    if (d->cardview_selected)
         g_list_free (d->cardview_selected);
-    }
 
     gtk_window_get_size (GTK_WINDOW(d->window), &w, &h);
     prefs_set_quiz_size (d->ig->prefs, w, h);
@@ -810,18 +872,6 @@ cb_window_delete (GtkWidget *widget, GdkEvent *event, Dialog *d)
 }
 
 
-static gboolean
-add_to_card_selection (GtkTreeModel *model,
-    GtkTreePath *path, GtkTreeIter *iter, Dialog *d)
-{
-    GList *item;
-
-    gtk_tree_model_get (model, iter, COLUMN_CARD_DATA, &item, -1);
-    d->cardview_selected = g_list_append (d->cardview_selected, CARD(item));
-    return TRUE;
-}
-
-
 static void
 toggle_ui_known (Dialog *d, gboolean active)
 {
@@ -900,16 +950,13 @@ update_title (void)
     Card *c;
     gchar *title, *flag = "";
 
-    if (!dialog) { return; }
+    if (!dialog)
+        return;
 
     c = QCARD(dialog->cur);
 
-    if (!CUR_IS_LAST(dialog)) {
-
-        if (card_get_tagged (c)) {
-          flag = UNICHAR_TAGGED;
-        }
-    }
+    if (!CUR_IS_LAST(dialog) && card_get_flagged (c))
+        flag = UNICHAR_FLAGGED;
 
     title = g_strdup_printf (_("Quiz %s"), flag);
 
@@ -923,7 +970,7 @@ void
 dialog_quiz (Ignuit *ig)
 {
     Dialog    *d;
-    GtkWidget *m_close;
+    GtkWidget *label, *m_close, *b_close_answer_bar;
     GladeXML  *glade_xml;
     gchar     *glade_file;
     Card      *c;
@@ -950,20 +997,7 @@ dialog_quiz (Ignuit *ig)
 
         /* Get a list of selected items from the main window's card treeview. */
 
-        GtkTreeSelection *sel;
-        GtkTreeModel *model;
-        GList *rows;
-
-        sel = gtk_tree_view_get_selection (d->ig->treev_card);
-        rows = gtk_tree_selection_get_selected_rows (sel, &model);
-        if (rows) {
-
-            gtk_tree_selection_selected_foreach (sel,
-                (GtkTreeSelectionForeachFunc)add_to_card_selection, d);
-
-            g_list_foreach (rows, (GFunc)gtk_tree_path_free, NULL);
-            g_list_free (rows);
-        }
+        d->cardview_selected = treev_get_selected_items (d->ig->treev_card);
 
     }
 
@@ -1003,9 +1037,9 @@ dialog_quiz (Ignuit *ig)
             card_set_front (c,
             _("No cards\n\nThere are no new or expired cards to test.\nSee the <i>Quiz</i> menu for options."));
             break;
-        case QUIZ_TAGGED_CARDS:
+        case QUIZ_FLAGGED_CARDS:
             card_set_front (c,
-            _("No cards\n\nThere are no cards tagged.\nSee the <i>Quiz</i> menu for options."));
+            _("No cards\n\nThere are no cards flagged.\nSee the <i>Quiz</i> menu for options."));
             break;
         case QUIZ_SELECTED_CARDS:
             card_set_front (c,
@@ -1070,6 +1104,8 @@ dialog_quiz (Ignuit *ig)
         (glade_xml, "m_flip");
     d->m_info = (GtkCheckMenuItem*)glade_xml_get_widget
         (glade_xml, "m_info");
+    d->m_answer_bar = (GtkCheckMenuItem*)glade_xml_get_widget
+        (glade_xml, "m_answer_bar");
     d->m_listen_front = (GtkWidget*)glade_xml_get_widget
         (glade_xml, "m_listen_front");
     d->m_listen_back = (GtkWidget*)glade_xml_get_widget
@@ -1078,10 +1114,16 @@ dialog_quiz (Ignuit *ig)
         (glade_xml, "m_auto_listen_front");
     d->m_auto_listen_back = (GtkCheckMenuItem*)glade_xml_get_widget
         (glade_xml, "m_auto_listen_back");
-    d->m_tag = (GtkCheckMenuItem*)glade_xml_get_widget
-        (glade_xml, "m_tag");
+    d->m_flag = (GtkCheckMenuItem*)glade_xml_get_widget
+        (glade_xml, "m_flag");
     m_close = glade_xml_get_widget (glade_xml, "m_close");
 
+
+    d->hbox_answer_bar = glade_xml_get_widget (glade_xml, "hbox_answer_bar");
+    d->entry_answer_bar = glade_xml_get_widget (glade_xml, "entry_answer_bar");
+    d->b_clear_answer_bar = glade_xml_get_widget (glade_xml, "b_clear_answer_bar");
+    b_close_answer_bar = glade_xml_get_widget (glade_xml, "b_close_answer_bar");
+
     d->b_info = (GtkToggleButton*)glade_xml_get_widget (glade_xml, "b_info");
     d->b_prev = glade_xml_get_widget (glade_xml, "b_prev");
     d->b_next = glade_xml_get_widget (glade_xml, "b_next");
@@ -1093,6 +1135,9 @@ dialog_quiz (Ignuit *ig)
     d->b_flip = (GtkToggleButton*)glade_xml_get_widget
         (glade_xml, "b_flip");
 
+    label = glade_xml_get_widget (glade_xml, "label_answer_bar");
+    gtk_label_set_mnemonic_widget (GTK_LABEL(label), d->entry_answer_bar);
+
     d->statusbar = (GtkStatusbar*)glade_xml_get_widget
         (glade_xml, "statusbar");
     d->sbcid = gtk_statusbar_get_context_id (d->statusbar, "quiz");
@@ -1123,15 +1168,25 @@ dialog_quiz (Ignuit *ig)
     g_signal_connect (G_OBJECT(d->m_auto_listen_back), "toggled",
         G_CALLBACK(cb_m_auto_listen), d);
 
-    g_signal_connect (G_OBJECT(d->m_tag), "toggled",
-        G_CALLBACK(cb_m_tag), d);
+    g_signal_connect (G_OBJECT(d->m_flag), "toggled",
+        G_CALLBACK(cb_m_flag), d);
 
     g_signal_connect (G_OBJECT(d->m_info), "toggled",
         G_CALLBACK(cb_m_info), d);
 
+    g_signal_connect (G_OBJECT(d->m_answer_bar), "toggled",
+        G_CALLBACK(cb_m_answer_bar), d);
+
     g_signal_connect (G_OBJECT(m_close), "activate",
         G_CALLBACK(cb_m_close), d);
 
+    g_signal_connect (G_OBJECT(b_close_answer_bar), "clicked",
+        G_CALLBACK(cb_b_close_answer_bar), d);
+    g_signal_connect (G_OBJECT(d->b_clear_answer_bar), "clicked",
+        G_CALLBACK(cb_b_clear_answer_bar), d);
+    g_signal_connect (G_OBJECT(d->entry_answer_bar), "activate",
+        G_CALLBACK(cb_answer_bar_activate), d);
+
     g_signal_connect (G_OBJECT(d->b_info), "toggled",
         G_CALLBACK(cb_b_info), d);
     g_signal_connect (G_OBJECT(d->b_prev), "clicked",
@@ -1155,6 +1210,12 @@ dialog_quiz (Ignuit *ig)
     g_signal_handlers_block_by_func (G_OBJECT(d->m_auto_listen_back),
         G_CALLBACK(cb_m_auto_listen), d);
 
+    gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(d->m_answer_bar),
+        prefs_get_quiz_answer_bar_visible (d->ig->prefs));
+
+    if (!prefs_get_quiz_answer_bar_visible (d->ig->prefs))
+        gtk_widget_hide (d->hbox_answer_bar);
+
     gtk_check_menu_item_set_active (d->m_auto_listen_front,
         prefs_get_auto_listen (d->ig->prefs) & LISTEN_FRONT);
     gtk_check_menu_item_set_active (d->m_auto_listen_back,
