@@ -490,12 +490,12 @@ write_card_xml (FILE *fp, Card *c)
 
     text = card_get_front (c);
     s = g_markup_printf_escaped ("><front>%s</front>", text);
-    fprintf (fp, s);
+    fprintf (fp, "%s", s);
     g_free (s);
 
     text = card_get_back (c);
     s = g_markup_printf_escaped ("<back>%s</back></card>\n", text);
-    fprintf (fp, s);
+    fprintf (fp, "%s", s);
     g_free (s);
 }
 
@@ -607,7 +607,7 @@ fileio_save (Ignuit *ig, const gchar *fname, GError **err)
 
     f_strm = fopen (fname, "w");
     if (!f_strm) {
-        g_set_error (err, 0, 0, g_strerror (errno));
+        g_set_error (err, 0, 0, "%s", g_strerror (errno));
         return FALSE;
     }
 
@@ -860,7 +860,7 @@ fileio_export_xml (Ignuit *ig, const gchar *fname, const gchar *filter,
 
     f_strm_out = fopen (fname, "w");
     if (!f_strm_out) {
-        g_set_error (err, 0, 0, g_strerror (errno));
+        g_set_error (err, 0, 0, "%s", g_strerror (errno));
         return FALSE;
     }
 
